@@ -35,7 +35,7 @@ async fn test_read_write_transaction() -> Result<(), anyhow::Error> {
     let client = Client::new(DATABASE).await.context("error")?;
     let result: Result<(Option<Timestamp>, i64), RunInTxError> = client
         .read_write_transaction(
-            |tx| {
+            |tx, _cancel| {
                 let user_id= user_id.to_string();
                 Box::pin(async move {
                     let ms = vec![create_user_mutation("user_client_1x", &now), create_user_mutation("user_client_2x", &now)];
@@ -56,15 +56,11 @@ async fn test_read_write_transaction() -> Result<(), anyhow::Error> {
     let ts = Utc.timestamp(value.seconds, value.nanos as u32);
 
     let mut ro = client.read_only_transaction().await?;
-    let record = ro
-        .read("User", &user_columns(), Key::key(&"user_client_1x"))
-        .await?;
+    let record = ro.read("User", &user_columns(), Key::key(&"user_client_1x")).await?;
     let row = all_rows(record).await.pop().unwrap();
     assert_user_row(&row, "user_client_1x", &now, &ts);
 
-    let record = ro
-        .read("User", &user_columns(), Key::key(&"user_client_2x"))
-        .await?;
+    let record = ro.read("User", &user_columns(), Key::key(&"user_client_2x")).await?;
     let row = all_rows(record).await.pop().unwrap();
     assert_user_row(&row, "user_client_2x", &now, &ts);
 
@@ -84,10 +80,7 @@ async fn test_apply() -> Result<(), anyhow::Error> {
     let users: Vec<String> = (0..2).map(|x| format!("user_client_{}", x)).collect();
     let client = Client::new(DATABASE).await.context("error")?;
     let now = Utc::now();
-    let ms = users
-        .iter()
-        .map(|id| create_user_mutation(id, &now))
-        .collect();
+    let ms = users.iter().map(|id| create_user_mutation(id, &now)).collect();
     let value = client.apply(ms).await.unwrap().unwrap();
     let ts = Utc.timestamp(value.seconds, value.nanos as u32);
 
@@ -107,10 +100,7 @@ async fn test_apply_at_least_once() -> Result<(), anyhow::Error> {
     let users: Vec<String> = (0..2).map(|x| format!("user_client_x_{}", x)).collect();
     let client = Client::new(DATABASE).await.context("error")?;
     let now = Utc::now();
-    let ms = users
-        .iter()
-        .map(|id| create_user_mutation(id, &now))
-        .collect();
+    let ms = users.iter().map(|id| create_user_mutation(id, &now)).collect();
     let value = client.apply_at_least_once(ms).await.unwrap().unwrap();
     let ts = Utc.timestamp(value.seconds, value.nanos as u32);
 
@@ -134,8 +124,7 @@ async fn test_partitioned_update() -> Result<(), anyhow::Error> {
         .await
         .unwrap();
     let client = Client::new(DATABASE).await.context("error")?;
-    let stmt =
-        Statement::new("UPDATE User SET NullableString = 'aaa' WHERE NullableString IS NOT NULL");
+    let stmt = Statement::new("UPDATE User SET NullableString = 'aaa' WHERE NullableString IS NOT NULL");
     client.partitioned_update(stmt).await.unwrap();
 
     let mut single = client.single().await.unwrap();
