@@ -11,14 +11,10 @@ use std::ops::DerefMut;
 
 mod common;
 use common::*;
+
 use std::collections::HashMap;
 
-async fn assert_read(
-    tx: &mut ReadOnlyTransaction,
-    user_id: &str,
-    now: &DateTime<Utc>,
-    cts: &DateTime<Utc>,
-) {
+async fn assert_read(tx: &mut ReadOnlyTransaction, user_id: &str, now: &DateTime<Utc>, cts: &DateTime<Utc>) {
     let reader = match tx.read("User", &user_columns(), Key::key(&user_id)).await {
         Ok(tx) => tx,
         Err(status) => panic!("read error {:?}", status),
@@ -29,12 +25,7 @@ async fn assert_read(
     assert_user_row(&row, user_id, now, cts);
 }
 
-async fn assert_query(
-    tx: &mut ReadOnlyTransaction,
-    user_id: &str,
-    now: &DateTime<Utc>,
-    cts: &DateTime<Utc>,
-) {
+async fn assert_query(tx: &mut ReadOnlyTransaction, user_id: &str, now: &DateTime<Utc>, cts: &DateTime<Utc>) {
     let mut stmt = Statement::new("SELECT * FROM User WHERE UserId = @UserID");
     stmt.add_param("UserId", &user_id);
     let mut rows = execute_query(tx, stmt).await;
@@ -123,39 +114,25 @@ async fn test_complex_query() {
     assert_eq!(first_item.user_id, user_id_1);
     assert_eq!(first_item.item_id, 2);
     assert_eq!(first_item.quantity, 100);
-    assert_ne!(
-        DateTime::<Utc>::from(first_item.updated_at).to_string(),
-        now.to_string()
-    );
+    assert_ne!(DateTime::<Utc>::from(first_item.updated_at).to_string(), now.to_string());
     let second_item = user_items.pop().unwrap();
     assert_eq!(second_item.user_id, user_id_1);
     assert_eq!(second_item.item_id, 1);
     assert_eq!(second_item.quantity, 100);
-    assert_ne!(
-        DateTime::<Utc>::from(second_item.updated_at).to_string(),
-        now.to_string()
-    );
+    assert_ne!(DateTime::<Utc>::from(second_item.updated_at).to_string(), now.to_string());
     assert!(user_items.is_empty());
 
-    let mut user_characters = row
-        .column_by_name::<Vec<UserCharacter>>("UserCharacter")
-        .unwrap();
+    let mut user_characters = row.column_by_name::<Vec<UserCharacter>>("UserCharacter").unwrap();
     let first_character = user_characters.pop().unwrap();
     assert_eq!(first_character.user_id, user_id_1);
     assert_eq!(first_character.character_id, 20);
     assert_eq!(first_character.level, 1);
-    assert_ne!(
-        DateTime::<Utc>::from(first_character.updated_at).to_string(),
-        now.to_string()
-    );
+    assert_ne!(DateTime::<Utc>::from(first_character.updated_at).to_string(), now.to_string());
     let second_character = user_characters.pop().unwrap();
     assert_eq!(second_character.user_id, user_id_1);
     assert_eq!(second_character.character_id, 10);
     assert_eq!(second_character.level, 1);
-    assert_ne!(
-        DateTime::<Utc>::from(second_character.updated_at).to_string(),
-        now.to_string()
-    );
+    assert_ne!(DateTime::<Utc>::from(second_character.updated_at).to_string(), now.to_string());
     assert!(user_characters.is_empty());
 }
 
@@ -183,16 +160,11 @@ async fn test_batch_partition_query_and_read() {
         .collect();
     let cr2 = replace_test_data(session.deref_mut(), many).await.unwrap();
 
-    let mut tx = match BatchReadOnlyTransaction::begin(
-        session,
-        TimestampBound::strong_read(),
-        CallOptions::default(),
-    )
-    .await
-    {
-        Ok(tx) => tx,
-        Err(status) => panic!("begin error {:?}", status),
-    };
+    let mut tx =
+        match BatchReadOnlyTransaction::begin(session, TimestampBound::strong_read(), CallOptions::default()).await {
+            Ok(tx) => tx,
+            Err(status) => panic!("begin error {:?}", status),
+        };
 
     let ts = cr.commit_timestamp.as_ref().unwrap();
     let ts = Utc.timestamp(ts.seconds, ts.nanos as u32);
@@ -255,18 +227,10 @@ async fn test_many_records_struct() {
     let user_id = "user_x_6";
     let mutations = vec![create_user_mutation(user_id, &now)];
     let _ = replace_test_data(&mut session, mutations).await.unwrap();
-    let item_mutations = (0..5000)
-        .map(|x| create_user_item_mutation(user_id, x))
-        .collect();
-    let _ = replace_test_data(&mut session, item_mutations)
-        .await
-        .unwrap();
-    let characters_mutations = (0..5000)
-        .map(|x| create_user_character_mutation(user_id, x))
-        .collect();
-    let _ = replace_test_data(&mut session, characters_mutations)
-        .await
-        .unwrap();
+    let item_mutations = (0..5000).map(|x| create_user_item_mutation(user_id, x)).collect();
+    let _ = replace_test_data(&mut session, item_mutations).await.unwrap();
+    let characters_mutations = (0..5000).map(|x| create_user_character_mutation(user_id, x)).collect();
+    let _ = replace_test_data(&mut session, characters_mutations).await.unwrap();
 
     let mut tx = read_only_transaction(session).await;
     let mut stmt = Statement::new(
@@ -282,9 +246,7 @@ async fn test_many_records_struct() {
     let row = rows.pop().unwrap();
     let items = row.column_by_name::<Vec<UserItem>>("UserItem").unwrap();
     assert_eq!(5000, items.len());
-    let characters = row
-        .column_by_name::<Vec<UserCharacter>>("UserCharacter")
-        .unwrap();
+    let characters = row.column_by_name::<Vec<UserCharacter>>("UserCharacter").unwrap();
     assert_eq!(5000, characters.len());
 }
 
@@ -298,10 +260,7 @@ async fn test_read_row() {
     let _ = replace_test_data(&mut session, mutations).await.unwrap();
 
     let mut tx = read_only_transaction(session).await;
-    let row = tx
-        .read_row("User", &["UserId"], Key::key(&user_id))
-        .await
-        .unwrap();
+    let row = tx.read_row("User", &["UserId"], Key::key(&user_id)).await.unwrap();
     assert!(row.is_some())
 }
 
@@ -320,11 +279,7 @@ async fn test_read_multi_row() {
 
     let mut tx = read_only_transaction(session).await;
     let row = tx
-        .read(
-            "User",
-            &["UserId"],
-            vec![Key::key(&user_id), Key::key(&user_id2)],
-        )
+        .read("User", &["UserId"], vec![Key::key(&user_id), Key::key(&user_id2)])
         .await
         .unwrap();
     assert_eq!(2, all_rows(row).await.len());
